/***

The MIT License

Copyright (c) 2018 Teppei Tamra (TAM)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

***/

#include "saoriapplication.h"
#include "saoridef.h"
#include <QCryptographicHash>
#include <QDir>
#include <QStandardPaths>
#include <QDebug>

SaoriApplication *SaoriApplication::m_self = nullptr;

SaoriApplication::SaoriApplication(int &argc, char **argv)
    :QApplication(argc, argv, true)
{
    m_self = this;
    setOrganizationName(SAORI_QSETTINGS_ORGANIZATION);
    setOrganizationDomain(SAORI_QSETTINGS_DOMAIN);
    setApplicationName(SAORI_QSETTINGS_APPLICATION);

    m_config = new QSettings(SAORI_QSETTINGS_DOMAIN,SAORI_QSETTINGS_APPLICATION,this);

    loadSettings();

    QDir dir;
    if (!dir.mkpath(QStandardPaths::writableLocation(QStandardPaths::CacheLocation))) {
        // CacheLocation is not writable.
    }
}

SaoriApplication::~SaoriApplication()
{
    saveInstancesSettings();
    saveAccountsSettings();
}

Saoridon *SaoriApplication::findInstance(QUrl instance)
{
    for(int i = 0;i < getInstanceList()->count();i ++) {
        if (getInstanceList()->at(i)->instance() == instance) return getInstanceList()->at(i);
    }
    return nullptr;
}

SaoriAccount *SaoriApplication::findAccount(QString account)
{
    for(int i = 0;i < getAccountList()->count();i ++) {
        if (getAccountList()->at(i)->name() == account) return getAccountList()->at(i);
    }
    return nullptr;
}

QList<Saoridon *> *SaoriApplication::getInstanceList()
{
    return &m_self->m_instanceList;
}

QList<SaoriAccount *> *SaoriApplication::getAccountList()
{
    return &m_self->m_accountList;
}

SaoriApplication *SaoriApplication::saori()
{
    return m_self;
}

void SaoriApplication::loadSettings()
{
    m_config->beginGroup("instances");
    auto instances = m_config->childGroups();
    for(int i = 0;i < instances.count();i ++) {
        m_config->beginGroup(instances[i]);
        auto *instance = new Saoridon(m_config->value("url").toUrl(),this);
        instance->setClientId(m_config->value("clientId").toString());
        instance->setClientSecret(m_config->value("clientSecret").toString());
        m_instanceList.append(instance);
        m_config->endGroup();
    }
    m_config->endGroup();

    m_config->beginGroup("accounts");
    auto accounts = m_config->childGroups();
    for(int i = 0;i < accounts.count();i ++) {
        m_config->beginGroup(accounts[i]);
        auto *account = new SaoriAccount(accounts[i],
                                         findInstance(m_config->value("instance").toUrl()),
                                         m_config->value("accessToken").toString(),
                                         this);
        m_accountList.append(account);
        m_config->endGroup();
    }
    m_config->endGroup();

}

void SaoriApplication::saveInstancesSettings()
{
    m_config->remove("instances");
    m_config->beginGroup("instances");
    for(int i = 0;i < m_instanceList.count();i ++) {
        m_config->beginGroup(QCryptographicHash::hash(m_instanceList[i]->instance().toString().toLatin1(),QCryptographicHash::Md5));
        m_config->setValue("url",m_instanceList[i]->instance());
        m_config->setValue("clientId",m_instanceList[i]->clientId());
        m_config->setValue("clientSecret",m_instanceList[i]->clientSecret());
        m_config->endGroup();
    }
    m_config->endGroup();
    m_config->sync();
}

void SaoriApplication::saveAccountsSettings()
{
    m_config->remove("accounts");
    m_config->beginGroup("accounts");
    for(int i = 0;i < m_accountList.count();i ++) {
        m_config->beginGroup(m_accountList[i]->name());
        m_config->setValue("instance",m_accountList[i]->instance()->instance());
        m_config->setValue("accessToken",m_accountList[i]->accessToken());
        m_config->endGroup();
    }
    m_config->endGroup();
    m_config->sync();
}
